/*************************************************************************
* Title:    I2C master library using hardware TWI interface
* Author:   Peter Fleury <pfleury@gmx.ch>  http://jump.to/fleury
* File:     $Id: twimaster.c,v 1.3 2005/07/02 11:14:21 Peter Exp $
* Software: AVR-GCC 3.4.3 / avr-libc 1.2.3
* Target:   any AVR device with hardware TWI 
* Usage:    API compatible with I2C Software Library i2cmaster.h
**************************************************************************/
#include <inttypes.h>
#include <compat/twi.h>

#include "i2cmaster.h"
#include <avr/io.h>

#include <util/delay.h>

#include "led.h"

// Port for the I2C
#define I2C_DDR DDRB
#define I2C_PIN PINB
#define I2C_PORT PORTB

// Pins to be used in the bit banging
#define I2C_CLK 5	// PIN7 at 10-Pin Header
#define I2C_DAT 4	// PIN9 at 10-Pin Header

					// GND 8/10 at 10-Pin Header
					// VCC 2    at 10-Pin Header

#define I2C_DATA_HI() I2C_DDR &= ~( 1 << I2C_DAT );
#define I2C_DATA_LO() I2C_DDR |=  ( 1 << I2C_DAT );

#define I2C_CLOCK_HI() I2C_DDR &= ~( 1 << I2C_CLK );
#define I2C_CLOCK_LO() I2C_DDR |=  ( 1 << I2C_CLK );


#define delay 1.25


static void I2C_WriteBit( unsigned char c )
{
    if ( c > 0 )
    {
        I2C_DATA_HI();
    }
    else
    {
        I2C_DATA_LO();
    }
	_delay_us(delay);
    I2C_CLOCK_HI();
    _delay_us(delay);

    I2C_CLOCK_LO();
    _delay_us(delay);

    if ( c > 0 )
    {
        I2C_DATA_LO();
    }

    _delay_us(delay);

}

static unsigned char I2C_ReadBit(void)
{
    unsigned char c;
    I2C_DATA_HI();
	_delay_us(delay);

    I2C_CLOCK_HI();
    _delay_us(delay);

    c = I2C_PIN;

    I2C_CLOCK_LO();
    _delay_us(delay);

    return ( c >> I2C_DAT ) & 1;
}

// Inits bitbanging port, must be called before using the functions below
//
static void I2C_Init(void)
{
    //ledGreenOn();
	I2C_PORT &= ~( ( 1 << I2C_DAT ) | ( 1 << I2C_CLK ) );

    I2C_CLOCK_HI();
    I2C_DATA_HI();

    _delay_us(delay);
}

// Send a START Condition
//
static void I2C_Start(void)
{
    ledRedOn();
	// set both to high at the same time
    I2C_DDR &=  ~( ( 1 << I2C_DAT ) | ( 1 << I2C_CLK ) );
    _delay_us(delay);

    I2C_DATA_LO();
    _delay_us(delay);

    I2C_CLOCK_LO();
    _delay_us(delay);
}

// Send a STOP Condition
//
static void I2C_Stop(void)
{
	
	_delay_us(delay);
	I2C_DATA_LO();
    _delay_us(delay);

	I2C_CLOCK_HI();
    _delay_us(delay);

    I2C_DATA_HI();
    _delay_us(delay);
    ledRedOff();
}

// write a byte to the I2C slave device
//
static unsigned char I2C_Write( unsigned char c )
{
    char i;
    for (i=0;i<8;i++)
    {
        I2C_WriteBit( c & 128 );

     c<<=1;
    }

    return I2C_ReadBit();
}


// read a byte from the I2C slave device
//
static unsigned char I2C_Read( unsigned char ack )
{
    unsigned char res = 0;
    char i;

    for ( i=0;i<8;i++)
    {
        res <<= 1;
        res |= I2C_ReadBit();
    }

    if ( ack > 0)
    {
        I2C_WriteBit( 0 );
    }
    else
    {
        I2C_WriteBit( 1 );
    }

    _delay_us(delay);

    return res;
}









/*************************************************************************
 Initialization of the I2C bus interface. Need to be called only once
*************************************************************************/
void i2c_init(void)
{
	I2C_Init();

}/* i2c_init */


/*************************************************************************
  Issues a start condition and sends address and transfer direction.
  return 0 = device accessible, 1= failed to access device
*************************************************************************/
unsigned char i2c_start(unsigned char address)
{
	I2C_Start();
	return I2C_Write(address);
}/* i2c_start */




/*************************************************************************
 Terminates the data transfer and releases the I2C bus
*************************************************************************/
void i2c_stop(void)
{
	I2C_Stop();
}/* i2c_stop */


/*************************************************************************
  Send one byte to I2C device
  
  Input:    byte to be transfered
  Return:   0 write successful 
            1 write failed
*************************************************************************/
unsigned char i2c_write( unsigned char data )
{	
	return I2C_Write(data);
}/* i2c_write */


/*************************************************************************
 Read one byte from the I2C device, request more data from device 
 
 Return:  byte read from I2C device
*************************************************************************/
unsigned char i2c_readAck(void)
{
	return I2C_Read(1);
}/* i2c_readAck */


/*************************************************************************
 Read one byte from the I2C device, read is followed by a stop condition 
 
 Return:  byte read from I2C device
*************************************************************************/
unsigned char i2c_readNak(void)
{
	return I2C_Read(0);
}/* i2c_readNak */


