/* Name: main.c
 * Project: custom-class, a basic USB example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-09
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 790 2010-05-30 21:00:26Z cs $
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.
We assume that an LED is connected to port B bit 0. If you connect it to a
different port or bit, change the macros below:
*/


#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */
#include "requests.h"       /* The custom request numbers we use */


#include "i2cmaster.h"		/* i2c libary by Peter Fleury (modified by skriptkiddy - bitbanging) */

#include "led.h"


#include "eeprom.h"
#include "spi.h"
#include "93Cxx.h"

static volatile uint32_t	led_reset_counter=0;


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

static uchar bytesRemaining;


static uint8_t   i2c_addr;
static uint16_t  address_offset;
static uint8_t   address_16_bit;


uchar usbFunctionRead(uchar *data, uchar len)
{
    if(len > bytesRemaining)                // len is max chunk size
        len = bytesRemaining;               // send an incomplete chunk
    bytesRemaining -= len;

    eeprom_read_block(i2c_addr, data, address_offset, len, address_16_bit);

    address_offset += len;



    return len;                             // return real chunk size
}



uchar usbFunctionWrite(uchar *data, uchar len)
{
    if(len > bytesRemaining)                // if this is the last incomplete chunk
        len = bytesRemaining;               // limit to the amount we can store
    bytesRemaining -= len;
    /* write 8 byte blocks to  eeprom */

    eeprom_write_pagewise(i2c_addr, data, len, 8, address_offset, address_16_bit);

    address_offset += len;



    return bytesRemaining == 0;             // return 1 if we have all data
}




usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

uint16_t	temp;

static uchar    dataBuffer[4];  /* buffer must stay valid when usbFunctionSetup returns */
	led_reset_counter = 10000;
	if(rq->bRequest == CUSTOM_RQ_ECHO){ /* echo -- used for reliability tests */
		dataBuffer[0] = rq->wValue.bytes[0];
		dataBuffer[1] = rq->wValue.bytes[1];
		dataBuffer[2] = rq->wIndex.bytes[0];
		dataBuffer[3] = rq->wIndex.bytes[1];
		usbMsgPtr = dataBuffer;         /* tell the driver which data to return */
		return 4;
	} else if(rq->bRequest == EEPROM_WRITE_24C) {
		address_offset = rq->wIndex.word;
		i2c_addr = (rq->wValue.word >> 8) & 0xff;
		address_16_bit = rq->wValue.word & 0xff;
		bytesRemaining = rq->wLength.word;
		return USB_NO_MSG;

	} else if(rq->bRequest == EEPROM_READ_24C) {
		address_offset = rq->wIndex.word;
		i2c_addr = (rq->wValue.word >> 8) & 0xff;
		address_16_bit = rq->wValue.word & 0xff;
		bytesRemaining = rq->wLength.word;
		return USB_NO_MSG;
	} else if(rq->bRequest == EEPROM_MODE_SET) {
		if (rq->wValue.word == EEPROM_MODE_24C) {
			i2c_init();
		} else if (rq->wValue.word == EEPROM_MODE_24C){
			spi_init();
		}
		return 0;
	} else if (rq->bRequest == EEPROM_READ_93C) {
		spi_init();
		if (rq->wIndex.word >> 8 != 0) { // 8bit address
			temp = eeprom_read_add8_bit(rq->wIndex.word & 0xff);
			dataBuffer[0] = (temp >> 0) & 0xff;
			dataBuffer[1] = (temp >> 8) & 0xff;
		} else {
			temp = eeprom_read_add6_bit(rq->wIndex.word & 0xff);
			dataBuffer[0] = (temp >> 0) & 0xff;
			dataBuffer[1] = (temp >> 8) & 0xff;
		}
		usbMsgPtr = dataBuffer;
		return 2;
	} else if (rq->bRequest == EEPROM_WRITE_93C) {
		spi_init();
		if (rq->wIndex.word >> 8 != 0) { // 8bit address
			eeprom_write_add_8bit(rq->wIndex.word, rq->wValue.word);
		} else {
			eeprom_write_add_6bit(rq->wIndex.word, rq->wValue.word);
		}
		return 0;
	}

	return 0;   /* default for not implemented requests: return no data back to host */
}



/* -------------------------------------------------------------------------- */
/* --------------------------------- Main ----------------------------------- */
/* -------------------------------------------------------------------------- */

int main(void)
{
uchar   i;
	ledInit();
    i2c_init();
	wdt_enable(WDTO_1S);
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
    odDebugInit();
    DBG1(0x00, 0, 0);       /* debug output: main starts */
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();

    sei();
    DBG1(0x01, 0, 0);       /* debug output: main loop starts */
    for(;;){                /* main event loop */
        DBG1(0x02, 0, 0);   /* debug output: main loop iterates */
        wdt_reset();
        usbPoll();
        _delay_us(1);
        if (led_reset_counter) led_reset_counter--;
        else  {
        	ledGreenOff();
        	ledRedOff();
        }
    }

   return 0;
}

/* ------------------------------------------------------------------------- */
