/* 
 * Copyright (C) 2013 Oliver J 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdio.h>
#include <string.h>
#include <inttypes.h>
#include <sys/time.h>

#include "usb_device.h"
#include "eeprom.h"
#include "requests.h"
#include <usb.h>

#define ADDRESS_8_BIT	0
#define ADDRESS_16_BIT	1

#define CHUNK 16


static int eeprom_24C_read_block(uint8_t i2c_addr, uint8_t *buf, uint16_t offset, int length, uint8_t address_format);

static int eeprom_24C_write_pagewise(uint8_t i2c_addr, uint8_t *buf, int length, uint16_t offset, uint8_t address_format);

static int eeprom_set_mode(uint8_t mode);


uint64_t get_time_ms(void) {
	struct timeval t;

	gettimeofday(&t, NULL);
	return t.tv_sec*1000+t.tv_usec/1000;
}


static uint64_t clock_start;

static uint32_t total_length;
static uint8_t *base_pointer;

#define time_start()     clock_start = get_time_ms()
#define time_elapsed()   (get_time_ms() - clock_start)



void update_progress_tty (int percent, double etime, char *hdr)
{
  static char hashes[51];
  static char *header;
  int i;



  hashes[50] = 0;

  if (percent > 100) percent = 100;


  memset (hashes, ' ', 50);
  for (i=0; i<percent; i+=2) {
    hashes[i/2] = '#';
  }

     header = hdr;


   fprintf(stdout, "\r%s | %s | %d%% %0.2fs",
           header, hashes, percent, etime);


  if (percent == 100) {
    fprintf (stdout, "\n\n");
  }


}



static int eeprom_24C_read_block(uint8_t i2c_addr, uint8_t *buf, uint16_t offset, int length, uint8_t address_format)
{
	int i;
	int retval;
	for (i=0; i<length; i=i+CHUNK) {
			retval = usb_control_msg(
								handle,
								USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
								EEPROM_READ_24C,
								(i2c_addr << 8) | address_format,
								offset+i,
								(char*) buf+i,
								CHUNK,
								1000);
			if (retval!=CHUNK) {
				fprintf(stderr, "error reading eeprom\n");
			}
			update_progress_tty(((int) (buf - base_pointer) + i + CHUNK)*100 / total_length, time_elapsed()/1000.0, "Reading: ");
		}
	return 0;
}



static int eeprom_24C_write_pagewise(uint8_t i2c_addr, uint8_t *buf, int length, uint16_t offset, uint8_t address_format) {

	int retval;
	int i;

	for (i=0; i<length; i=i+8) {
		retval = usb_control_msg(
							handle,
							USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT,
							EEPROM_WRITE_24C,
							(i2c_addr << 8) | address_format,
							offset+i,
							(char*) buf+i,
							8,
							1000);
		if (retval!=8) {
			fprintf(stderr, "error writing eeprom\n");
		}
		update_progress_tty(((int) (buf - base_pointer) + i + 8)*100 / total_length, time_elapsed()/1000.0, "Writing: ");
	}
	return 0;
}



int eeprom_24C_read(uint8_t i2c_base_addr , uint8_t *buffer , int EepromSize1K) {

	total_length = EepromSize1K * 128;
	base_pointer = buffer;
	time_start();
	if (EepromSize1K == 0) { /* Workaround for 24C00*/
		total_length = 16;
	}

	eeprom_set_mode(EEPROM_MODE_24C);

	if (EepromSize1K==1024) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x00000, 0x00, 0x10000, ADDRESS_16_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x01, buffer + 0x10000, 0x00, 0x10000, ADDRESS_16_BIT);
	}


	if (EepromSize1K==512) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x00000, 0x00, 0x10000, ADDRESS_16_BIT);
	}

	if (EepromSize1K==256) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x00000, 0x00, 0x08000, ADDRESS_16_BIT);
	}

	if (EepromSize1K==128) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x00000, 0x00, 0x04000, ADDRESS_16_BIT);
	}


	if (EepromSize1K==64) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x00000, 0x00, 0x02000, ADDRESS_16_BIT);
	}



	if (EepromSize1K==32) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x00000, 0x00, 0x01000, ADDRESS_16_BIT);
	}

	if (EepromSize1K==16) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x000, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x01, buffer + 0x100, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x02, buffer + 0x200, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x03, buffer + 0x300, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x04, buffer + 0x400, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x05, buffer + 0x500, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x06, buffer + 0x600, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x07, buffer + 0x700, 0x00, 0x100, ADDRESS_8_BIT);
	}


	if (EepromSize1K==8) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x000, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x01, buffer + 0x100, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x02, buffer + 0x200, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x03, buffer + 0x300, 0x00, 0x100, ADDRESS_8_BIT);
	}


	if (EepromSize1K==4) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x000, 0x00, 0x100, ADDRESS_8_BIT);
		eeprom_24C_read_block(i2c_base_addr + 0x01, buffer + 0x100, 0x00, 0x100, ADDRESS_8_BIT);
	}


	if (EepromSize1K==2) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x000, 0x00, 0x100, ADDRESS_8_BIT);
	}

	if (EepromSize1K==1) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x000, 0x00, 0x080, ADDRESS_8_BIT);
	}

	if (EepromSize1K==0) {
		eeprom_24C_read_block(i2c_base_addr + 0x00, buffer + 0x000, 0x00, 0x010, ADDRESS_8_BIT);
	}

	return 0;
}



int eeprom_24C_write(uint8_t i2c_base_addr , uint8_t *buffer , int EepromSize1K) {

	time_start();
	base_pointer = buffer;
	total_length = EepromSize1K * 128;

	if (EepromSize1K == 0) { /* Workaround for 24C00*/
		total_length = 16;
	}

	eeprom_set_mode(EEPROM_MODE_24C);

	if (EepromSize1K==1024) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x00000, 0x10000, 0, ADDRESS_16_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x01, buffer+0x10000, 0x10000, 0, ADDRESS_16_BIT);
	}

	if (EepromSize1K==512) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x00000, 0x10000, 0, ADDRESS_16_BIT);
	}


	if (EepromSize1K==256) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x00000, 0x08000, 0, ADDRESS_16_BIT);
	}



	if (EepromSize1K==128) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x00000, 0x04000, 0, ADDRESS_16_BIT);
	}


	if (EepromSize1K==64) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x00000, 0x02000, 0, ADDRESS_16_BIT);
	}


	if (EepromSize1K==32) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x00000, 0x01000, 0, ADDRESS_16_BIT);
	}


	if (EepromSize1K==16) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x000, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x01, buffer+0x100, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x02, buffer+0x200, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x03, buffer+0x300, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x04, buffer+0x400, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x05, buffer+0x500, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x06, buffer+0x600, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x07, buffer+0x700, 0x100, 0, ADDRESS_8_BIT);
	}


	if (EepromSize1K==8) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x000, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x01, buffer+0x100, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x02, buffer+0x200, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x03, buffer+0x300, 0x100, 0, ADDRESS_8_BIT);
	}


	if (EepromSize1K==4) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x000, 0x100, 0, ADDRESS_8_BIT);
		eeprom_24C_write_pagewise(i2c_base_addr + 0x01, buffer+0x100, 0x100, 0, ADDRESS_8_BIT);
	}


	if (EepromSize1K==2) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x000, 0x100, 0, ADDRESS_8_BIT);
	}


	if (EepromSize1K==1) {
		eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x000, 0x080, 0, ADDRESS_8_BIT);
	}


	if (EepromSize1K==0) {
			eeprom_24C_write_pagewise(i2c_base_addr + 0x00, buffer+0x000, 0x010, 0, ADDRESS_8_BIT);
		}


	return 0;
}



int eeprom_93C_read(uint8_t *buffer , int EepromSize) {
	int i, retval;

	uint16_t addr_8_bit_flag = 0;

	eeprom_set_mode(EEPROM_MODE_93C);

	time_start();

	if (EepromSize > 128) {
		addr_8_bit_flag = 1;
	}

	for (i=0; i<EepromSize; i=i+2) {
			retval = usb_control_msg(
								handle,
								USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
								EEPROM_READ_93C,
								0,
								i/2 | (addr_8_bit_flag << 8),
								(char*) buffer+i,
								2,
								1000);
			if (retval!=2) {
				perror("error:");
				fprintf(stderr, "error reading eeprom %i\n", retval);
			}
			update_progress_tty(((int) ( i + 2 ) )*100 / EepromSize, time_elapsed()/1000.0, "Reading: ");
		}
	return 0;
}


int eeprom_93C_write(uint8_t *buffer , int EepromSize)
{
	uint8_t addr_8_bit_flag = 0;

	time_start();

	if (EepromSize > 128) {
			addr_8_bit_flag = 1;
		}
	for (int i=0; i<EepromSize; i=i+2) {
				int retval = usb_control_msg(
									handle,
									USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT,
									EEPROM_WRITE_93C,
									buffer[i] + 0x100 * buffer[i+1],
									i/2 | (addr_8_bit_flag << 8),
									NULL,
									0,
									1000);
				if (retval!=0) {
					fprintf(stderr, "error writing eeprom\n");
				}
				update_progress_tty(((int) ( i + 2 ) )*100 / EepromSize, time_elapsed()/1000.0, "Writing: ");
			}
	return 0;
}


static int eeprom_set_mode(uint8_t mode) {
	int retval = usb_control_msg(
					handle,
					USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT,
					EEPROM_MODE_SET,
					mode,
					0,
					NULL,
					0,
					1000);
				if (retval!=0) {
					fprintf(stderr, "error set EEPROM mode\n");
				}

	return 0;
}

