/* 
 * Copyright (C) 2013 Oliver J 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <getopt.h>
#include <error.h>

#include "usb_device.h"
#include "eeprom.h"
#include "hexdump.h"


#define EEPROM_BASE_ADDR 	0x50
#define SIZE 				32UL
#define ANZ 				128*SIZE

#define MAX_SIZE			1024

char *progname = NULL;


typedef struct {
	char name[8];
	int  size_k;
	int  size_b;
	enum {_24C, _93C, _NO} type;
} eeprom_t;

eeprom_t eeproms[] = {
	{"24C00", 	0, 		16  * 1		,	_24C},
	{"24C01", 	1, 		128 * 1		,	_24C},
	{"24C02", 	2, 		128 * 2		,	_24C},
	{"24C04", 	4, 		128 * 4		,	_24C},
	{"24C08", 	8, 		128 * 8		,	_24C},
	{"24C16", 	16,	 	128 * 16	,	_24C},
	{"24C32", 	32, 	128 * 32	,	_24C},
	{"24C64", 	64, 	128 * 64	,	_24C},
	{"24C128", 	128, 	128 * 128	,	_24C},
	{"24C256", 	256, 	128 * 256	,	_24C},
	{"24C512", 	512, 	128 * 512	,	_24C},
	{"24C1024", 1024, 	128 * 1024	,	_24C},
	{"93C06", 	0, 		32  * 1		,	_93C},
	{"93C46", 	0, 		32  * 4		,	_93C},
	{"93C56", 	0, 		32  * 8		,	_93C},
	{"93C66", 	0, 		32  * 16	,	_93C},
};


void usage(void)
{
	fprintf(stderr,
			"Usage: %s [options]\n"
			"Options:\n"
			"  -w                     Write binary file to EEPROM (needs -f)\n"
			"  -r                     Read EEPROM content to binary file (needs -f)\n"
			"  -f <filename>          Binary file name\n"
			"  -t <type>              EEPROM type:\n"
			"                          24C00, 24C01, 24C02, 24C04, 24C08, 24C16, 24C32, \n"
			"                          24C64, 24C128, 24C256, 24C512, 24C1024, \n"
			"                          93C06, 93C46, 93C56, 93C66\n"
			"  -d                     Prints EEPROM content as hexdump\n",

			progname);

}



int main(int argc, char * argv []){


	uint8_t *buf_write = NULL;
	uint8_t *buf_read = NULL;

	char* filename = NULL;
	uint8_t  eeprom_wr = 0;
	uint8_t  eeprom_rd = 0;
	uint8_t  print_hexdump = 0;

	eeprom_t current_eeprom={"", 0, 0, _NO};


	int ch;

	progname = argv[0];

	printf("\n");
	/* Parse command line Arguments */
	while ((ch = getopt(argc,argv,"rwf:t:hd")) != -1) {

		switch (ch) {
			case 'r':
				if (eeprom_wr) {
					usage();
					fprintf(stderr, "error: Could not read and write at the same time.\n\n");
					return -1;
				}
				eeprom_rd = 1;
				break;

			case 'w':
				if (eeprom_rd) {
					usage();
					fprintf(stderr, "error: Could not read and write at the same time.\n\n");
					return -1;
				}
				eeprom_wr = 1;
				break;

			case 't': {
				int ok = 0;
				/* iterate through eeprom-array and check if argument exists */
				for (int i=0; i < sizeof(eeproms)/sizeof(eeproms[0]); i++) {
					if (strcmp(eeproms[i].name, optarg) == 0) {
						current_eeprom = eeproms[i];
						ok = 1;
						break;
					}
				}
				if (ok == 0) {
					usage();
					fprintf(stderr, "error: Unknown EEPROM type.\n\n");
					return -1;
				}
				break;
			}

			case 'f':
				filename = optarg;
				break;

			case 'd':
				print_hexdump = 1;
				break;

			case 'h':
				usage();
				return 0;
				break;
		}
	}

	setvbuf(stdout, (char*)NULL, _IONBF, 0);


	if (filename == NULL && print_hexdump == 0) {
		fprintf(stderr, "error: You must specify a filename.\n\n");
		usage();
		return -1;
	}

	/* initialize connection to eeprog hardware */
	usb_device_init();


	if (eeprom_wr && filename != NULL) {

		/* Open file for reading out Data to be written to EEPROM */
		FILE *fp = fopen(filename, "rb");
		if (fp==NULL) {
			perror("error: ");
			return -1;
		}

		/* allocate Memory for write buffer */
		buf_write = malloc(current_eeprom.size_b);
		if (buf_write == NULL) {
			fprintf(stderr, "error: Cannot allocate enough memory.\n\n");
			return -1;
		}

		/* allocate Memory for read buffer */
		buf_read = malloc(current_eeprom.size_b);
		if (buf_read == NULL) {
			free(buf_write);
			fprintf(stderr, "error: Cannot allocate enough memory.\n\n");
			return -1;
		}

		/* Calculate filesize */
		uint32_t filesize = 0;
		fseek(fp, 0, SEEK_END);
		filesize = ftell(fp);
		fseek(fp, 0, SEEK_SET);

		/* compare filesize to EEPROM size*/
		if (filesize > current_eeprom.size_b) {
			fprintf(stderr, "warning: File to big to fit into a \"%s\". Will truncate data.\n\n", current_eeprom.name);
			filesize = current_eeprom.size_b;
		} else if (filesize < current_eeprom.size_b){
			fprintf(stderr, "warning: File to small for \"%s\". Filling empty space with zeros.\n\n", current_eeprom.name);
			memset(buf_write, 0, current_eeprom.size_b);
		}

		/* read in file */
		if (fread(buf_write, sizeof(uint8_t), current_eeprom.size_b, fp) != filesize) {
			fprintf(stderr, "error: could not read %s completely.\n", filename);
			free(buf_write);
			free(buf_read);
			return -1;
		}

		/* Write EERPROM and read out its content after that for later verification */
		if (current_eeprom.type == _24C) {
			eeprom_24C_write(EEPROM_BASE_ADDR,(uint8_t*)buf_write, current_eeprom.size_k);
			eeprom_24C_read (EEPROM_BASE_ADDR,(uint8_t*)buf_read,  current_eeprom.size_k);
		} else if (current_eeprom.type == _93C) {
			eeprom_93C_write(buf_write, current_eeprom.size_b);
			eeprom_93C_read (buf_read, current_eeprom.size_b);
		} else {
			fprintf(stderr, "error: no valid EEprom Type.\n");
			free(buf_read);
			free(buf_write);
			return -1;
		}

		/* Print Hexdump of EEPROM Content if wanted */
		if (print_hexdump) {
			hexdump_buf(0x0000, buf_read, current_eeprom.size_b);
			printf("\n");
		}

		/* Verify EEEPROM content */
		printf("Verifying %i bytes.\n\n", current_eeprom.size_b);
		uint8_t verify_failed = 0;
		for (int i=0; i < current_eeprom.size_b; i++) {
			if (buf_read[i] != buf_write[i]) {
				printf("Verify failed: First mismatch @ 0x%05x [ %02X != %02X ].\n\n", i, buf_write[i], buf_read[i]);
				verify_failed = 1;
				break;
			}
		}
		if (verify_failed == 0) {
			printf("Verify  ok.\n\n");
		}

		free(buf_write);
		free(buf_read);

	} else if (eeprom_rd && filename != NULL) {

		/* open file to store EEPROM content within */
		FILE *fp = fopen(filename, "wb");
		if (fp==NULL) {
			perror("error: ");
			return -1;
		}

		/* allocate Memory for read buffer */
		buf_read = malloc(current_eeprom.size_b);
		if (buf_read == NULL) {
			fprintf(stderr, "error: Cannot allocate enough memory \n");
			return -1;
		}

		/* Read out EEPROM */
		if (current_eeprom.type == _24C) {
			eeprom_24C_read (EEPROM_BASE_ADDR,(uint8_t*)buf_read,  current_eeprom.size_k);
		} else if (current_eeprom.type == _93C) {
			eeprom_93C_read (buf_write, current_eeprom.size_b);
		} else {
			fprintf(stderr, "error: no valid EEprom Type\n");
			free(buf_read);
			return -1;
		}

		/* Write EEPROM content to file*/
		fseek(fp, 0, SEEK_SET);
		if (fwrite(buf_read, sizeof(uint8_t), current_eeprom.size_b, fp) != current_eeprom.size_b) {
			fprintf(stderr, "error: could not write eeprom content to %s completely\n", filename);
			free(buf_read);
			return -1;
		}

		/* Print Hexdump of EEPROM Content if wanted */
		if (print_hexdump) {
			hexdump_buf(0x0000, buf_read, current_eeprom.size_b);
		}

		free(buf_read);

	} else if (print_hexdump) {

		/* allocate Memory for read buffer*/
		buf_read = malloc(current_eeprom.size_b);
		if (buf_read == NULL) {
			fprintf(stderr, "error: Cannot allocate enough memory \n");
			return -1;
		}

		/* Read out EEPROM */
		if (current_eeprom.type == _24C) {
			eeprom_24C_read (EEPROM_BASE_ADDR,(uint8_t*)buf_read,  current_eeprom.size_k);
		} else if (current_eeprom.type == _93C) {
			eeprom_93C_read (buf_read, current_eeprom.size_b);
		} else {
			fprintf(stderr, "error: no valid EEprom Type\n");
			free(buf_read);
			return -1;
		}

		/* Print Hexdump of EEPROM Content */
		hexdump_buf(0x0000, buf_read, current_eeprom.size_b);

		free(buf_read);
	}


    return 0;
}
